/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2003 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <string.h>

#include <bonobo/bonobo-shlib-factory.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <gtk/gtkentry.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkmenushell.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtkoptionmenu.h>
#include <gtk/gtktogglebutton.h>
#include <libgnomeui/gnome-font-picker.h>

#include <shell/evolution-config-control.h>

#define ENG_CONFIG_FACTORY_ID "OAFIID:GNOME_Evolution_NoGnome_ConfigFactory"
#define ENG_CONFIG_CONTROL_ID "OAFIID:GNOME_Evolution_NoGnome_ConfigControl"

#define GNOME_BROWSER_DIR        "/desktop/gnome/url-handlers/unknown"
#define GNOME_BROWSER_KEY        GNOME_BROWSER_DIR "/command"

#define GNOME_FONT_DIR           "/desktop/gnome/interface"
#define GNOME_FONT_KEY           GNOME_FONT_DIR "/font_name"

#define GNOME_PROXY_DIR          "/system/http_proxy"
#define GNOME_PROXY_ENABLE_KEY   GNOME_PROXY_DIR "/use_http_proxy"
#define GNOME_PROXY_HOST_KEY     GNOME_PROXY_DIR "/host"
#define GNOME_PROXY_PORT_KEY     GNOME_PROXY_DIR "/port"
#define GNOME_PROXY_USE_AUTH_KEY GNOME_PROXY_DIR "/use_authentication"
#define GNOME_PROXY_USER_KEY     GNOME_PROXY_DIR "/authentication_user"
#define GNOME_PROXY_PASSWORD_KEY GNOME_PROXY_DIR "/authentication_password"

/* Not actually *all* keys, since the browser gets handled separately */
static const char *all_keys[] = {
	GNOME_FONT_KEY, GNOME_PROXY_ENABLE_KEY,
	GNOME_PROXY_HOST_KEY, GNOME_PROXY_PORT_KEY, GNOME_PROXY_USE_AUTH_KEY,
	GNOME_PROXY_USER_KEY, GNOME_PROXY_PASSWORD_KEY
};
static const int num_keys = G_N_ELEMENTS (all_keys);

static struct {
	const char *name;
	const char *program;
	const char *extra_args;
} known_browsers[] = {
	{ N_("Epiphany"), "epiphany", "--new-window" },
	{ N_("Galeon"), "galeon", "--new-window" },
	{ N_("Konqueror"), "konqueror", NULL },
	{ N_("Mozilla"), "mozilla", NULL },
	{ N_("Netscape"), "netscape", NULL },
	{ N_("Opera"), "opera", NULL }
};
static const int num_browsers = G_N_ELEMENTS (known_browsers);
#define EVOLUTION_NOGNOME_OTHER_BROWSER (num_browsers + 1)

typedef struct {
	EvolutionConfigControl *control;
	GladeXML *xml;
	GConfClient *gconf;
	guint change_handler_id;

	GnomeFontPicker *theme_font_picker;

	GtkOptionMenu *browser_omenu;
	GtkEntry *browser_other_entry;
	GtkWidget *browser_other_label;

	GtkToggleButton *proxy_check;
	GtkWidget *proxy_table;
	GtkEntry *proxy_host_entry;
	GtkEntry *proxy_port_entry;
	gboolean proxy_use_auth;
	GtkEntry *proxy_user_entry;
	GtkEntry *proxy_password_entry;
} EvolutionNoGnomeData;

static void
control_destroy_cb (gpointer user_data, GObject *where_control_was)
{
	EvolutionNoGnomeData *eng = user_data;

	g_object_unref (eng->xml);

	g_signal_handler_disconnect (eng->gconf, eng->change_handler_id);
	g_object_unref (eng->gconf);

	g_free (eng);
}

static void
control_apply_cb (EvolutionConfigControl *control, gpointer user_data)
{
	EvolutionNoGnomeData *eng = user_data;
	const char *string;
	gboolean bool;
	int num;

	string = gnome_font_picker_get_font_name (eng->theme_font_picker);
	gconf_client_set_string (eng->gconf, GNOME_FONT_KEY, string, NULL);

	string = gtk_entry_get_text (eng->browser_other_entry);
	gconf_client_set_string (eng->gconf, GNOME_BROWSER_KEY, string, NULL);

	bool = gtk_toggle_button_get_active (eng->proxy_check);
	gconf_client_set_bool (eng->gconf, GNOME_PROXY_ENABLE_KEY, bool, NULL);

	if (bool) {
		string = gtk_entry_get_text (eng->proxy_host_entry);
		gconf_client_set_string (eng->gconf, GNOME_PROXY_HOST_KEY,
					 string, NULL);

		string = gtk_entry_get_text (eng->proxy_port_entry);
		num = atoi (string);
		gconf_client_set_int (eng->gconf, GNOME_PROXY_PORT_KEY,
				      num, NULL);

		string = gtk_entry_get_text (eng->proxy_user_entry);
		if (*string) {
			gconf_client_set_bool (eng->gconf,
					       GNOME_PROXY_USE_AUTH_KEY,
					       TRUE, NULL);

			gconf_client_set_string (eng->gconf,
						 GNOME_PROXY_USER_KEY,
						 string, NULL);

			string = gtk_entry_get_text (eng->proxy_password_entry);
			gconf_client_set_string (eng->gconf,
						 GNOME_PROXY_PASSWORD_KEY,
						 string, NULL);
		} else {
			gconf_client_set_bool (eng->gconf,
					       GNOME_PROXY_USE_AUTH_KEY,
					       FALSE, NULL);
		}
	}
}

static void
theme_font_set (GnomeFontPicker *gfp, const char *font, gpointer user_data)
{
	EvolutionNoGnomeData *eng = user_data;

	evolution_config_control_changed (eng->control);
}

static void
entry_changed (GtkEditable *editable, gpointer user_data)
{
	EvolutionNoGnomeData *eng = user_data;

	evolution_config_control_changed (eng->control);
}

static void
browser_state_update (EvolutionNoGnomeData *eng)
{
	int browser;
	char *command;

	browser = gtk_option_menu_get_history (eng->browser_omenu);
	if (browser == EVOLUTION_NOGNOME_OTHER_BROWSER) {
		gtk_widget_show (GTK_WIDGET (eng->browser_other_entry));
		gtk_widget_show (eng->browser_other_label);
	} else {
		gtk_widget_hide (GTK_WIDGET (eng->browser_other_entry));
		gtk_widget_hide (eng->browser_other_label);

		command = known_browsers[browser].extra_args ?
			g_strdup_printf ("%s %s \"%%s\"",
					 known_browsers[browser].program,
					 known_browsers[browser].extra_args) :
			g_strdup_printf ("%s \"%%s\"",
					 known_browsers[browser].program);
		gtk_entry_set_text (eng->browser_other_entry, command);
		g_free (command);
	}
}

static void
browser_omenu_changed (GtkOptionMenu *omenu, gpointer user_data)
{
	EvolutionNoGnomeData *eng = user_data;

	browser_state_update (eng);
	evolution_config_control_changed (eng->control);
}

static void
proxy_state_update (EvolutionNoGnomeData *eng)
{
	gtk_widget_set_sensitive (eng->proxy_table,
				  gtk_toggle_button_get_active (eng->proxy_check));
}

static void
proxy_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	EvolutionNoGnomeData *eng = user_data;

	proxy_state_update (eng);
	evolution_config_control_changed (eng->control);
}

static void
setup_browser_menu (EvolutionNoGnomeData *eng)
{
	const char *command, *program;
	GtkWidget *menu, *item;
	int i, program_len, selected = -1;

	command = gconf_client_get_string (eng->gconf, GNOME_BROWSER_KEY, NULL);
	if (command) {
		/* Find the command name not including args */
		gtk_entry_set_text (eng->browser_other_entry, command);
		program = strrchr (command, ' ');
		if (program) {
			while (program > command && program[-1] != '/')
				program--;
		} else {
			program = strrchr (command, '/');
			if (program)
				program++;
			else
				program = command;
		}
		program_len = strcspn (program, " ");
	} else
		program = NULL;

	/* Fill in the menu, look for a match for @program */
	menu = gtk_option_menu_get_menu (eng->browser_omenu);
	for (i = 0; i < num_browsers; i++) {
		item = gtk_menu_item_new_with_label (_(known_browsers[i].name));
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

		if (program && selected == -1 &&
		    !strncmp (program, known_browsers[i].program, program_len))
			selected = i;
	}

	/* Add the separator */
	item = gtk_menu_item_new ();
	gtk_widget_set_sensitive (item, FALSE);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

	/* And the "Other" item */
	item = gtk_menu_item_new_with_label (_("Other..."));
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

	if (command == NULL) {
		/* Fill in the default choice */
		gtk_option_menu_set_history (eng->browser_omenu, 0);
		g_signal_emit_by_name (eng->browser_omenu, "changed");
	} else if (selected == -1) {
		gtk_option_menu_set_history (eng->browser_omenu,
					     EVOLUTION_NOGNOME_OTHER_BROWSER);
	} else
		gtk_option_menu_set_history (eng->browser_omenu, selected);

	gtk_widget_show_all (menu);
	browser_state_update (eng);
}

static void
value_changed (GConfClient *client, const char *key,
	       GConfValue *value, gpointer user_data)
{
	EvolutionNoGnomeData *eng = user_data;
	const char *string;
	gboolean bool;
	int num;

	if (!strcmp (key, GNOME_FONT_KEY)) {
		string = gconf_value_get_string (value);
		gnome_font_picker_set_font_name (eng->theme_font_picker, string);
	} else if (!strcmp (key, GNOME_BROWSER_KEY)) {
		setup_browser_menu (eng);
	} else if (!strcmp (key, GNOME_PROXY_ENABLE_KEY)) {
		bool = gconf_value_get_bool (value);
		gtk_toggle_button_set_active (eng->proxy_check, bool);
		proxy_state_update (eng);
	} else if (!strcmp (key, GNOME_PROXY_HOST_KEY)) {
		string = gconf_value_get_string (value);
		gtk_entry_set_text (eng->proxy_host_entry, string);
	} else if (!strcmp (key, GNOME_PROXY_PORT_KEY)) {
		char *tmp;
		num = gconf_value_get_int (value);
		tmp = g_strdup_printf ("%d", num);
		gtk_entry_set_text (eng->proxy_port_entry, tmp);
		g_free (tmp);
	} else if (!strcmp (key, GNOME_PROXY_USE_AUTH_KEY)) {
		eng->proxy_use_auth = gconf_value_get_bool (value);
	} else if (!strcmp (key, GNOME_PROXY_USER_KEY)) {
		if (eng->proxy_use_auth) {
			string = gconf_value_get_string (value);
			gtk_entry_set_text (eng->proxy_user_entry, string);
		}
	} else if (!strcmp (key, GNOME_PROXY_PASSWORD_KEY)) {
		if (eng->proxy_use_auth) {
			string = gconf_value_get_string (value);
			gtk_entry_set_text (eng->proxy_password_entry, string);
		}
	}
}

static BonoboObject *
factory (BonoboGenericFactory *factory,
	 const char *component_id,
	 void *closure)
{
	EvolutionNoGnomeData *eng;
	GtkWidget *table;
	GConfValue *value;
	int i;

	eng = g_new0 (EvolutionNoGnomeData, 1);
	eng->control = g_object_new (EVOLUTION_TYPE_CONFIG_CONTROL, NULL);

	eng->xml = glade_xml_new (EVOLUTION_GLADEDIR "/evolution-nognome.glade", "nognome_table", NULL);
	if (!eng->xml) {
		GtkWidget *label;

		bonobo_object_unref (eng->control);
		label = gtk_label_new (_("Unable to load configuration UI."));
		gtk_widget_show (label);

		return BONOBO_OBJECT (evolution_config_control_new (label));
	}

	/* Put the (already parentless) glade widgets into the control */
	table = glade_xml_get_widget (eng->xml, "nognome_table");
	evolution_config_control_construct (eng->control, table);

	/* Set up variables */
	eng->theme_font_picker = (GnomeFontPicker *)
		glade_xml_get_widget (eng->xml, "theme_font_picker");

	eng->browser_omenu = (GtkOptionMenu *)
		glade_xml_get_widget (eng->xml, "browser_omenu");
	eng->browser_other_entry = (GtkEntry *)
		glade_xml_get_widget (eng->xml, "browser_other_entry");
	eng->browser_other_label =
		glade_xml_get_widget (eng->xml, "browser_other_label");

	eng->proxy_check = (GtkToggleButton *)
		glade_xml_get_widget (eng->xml, "proxy_check");
	eng->proxy_table = 
		glade_xml_get_widget (eng->xml, "proxy_table");
	eng->proxy_host_entry = (GtkEntry *)
		glade_xml_get_widget (eng->xml, "proxy_host_entry");
	eng->proxy_port_entry = (GtkEntry *)
		glade_xml_get_widget (eng->xml, "proxy_port_entry");
	eng->proxy_user_entry = (GtkEntry *)
		glade_xml_get_widget (eng->xml, "proxy_user_entry");
	eng->proxy_password_entry = (GtkEntry *)
		glade_xml_get_widget (eng->xml, "proxy_password_entry");

	/* Set up GConf notifications */
	eng->gconf = gconf_client_get_default ();
	eng->change_handler_id =
		g_signal_connect (eng->gconf, "value_changed",
				  G_CALLBACK (value_changed), eng);

	gconf_client_add_dir (eng->gconf, GNOME_FONT_DIR,
			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
	gconf_client_add_dir (eng->gconf, GNOME_BROWSER_DIR,
			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
	gconf_client_add_dir (eng->gconf, GNOME_PROXY_DIR,
			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

	/* Fill in fields */
	for (i = 0; i < num_keys; i++) {
		value = gconf_client_get (eng->gconf, all_keys[i], NULL);
		if (value)
			value_changed (eng->gconf, all_keys[i], value, eng);
	}
	setup_browser_menu (eng);

	/* Now connect to "changed"-type signals; we have to do this
	 * after filling in the fields, because EvolutionConfigControl
	 * will get confused if you call config_control_changed()
	 * before it's displayed.
	 */
	g_signal_connect (eng->theme_font_picker, "font_set",
			  G_CALLBACK (theme_font_set), eng);

	g_signal_connect (eng->browser_omenu, "changed",
			  G_CALLBACK (browser_omenu_changed), eng);
	g_signal_connect (eng->browser_other_entry, "changed",
			  G_CALLBACK (entry_changed), eng);

	g_signal_connect (eng->proxy_check, "toggled",
			  G_CALLBACK (proxy_toggled), eng);
	g_signal_connect (eng->proxy_host_entry, "changed",
			  G_CALLBACK (entry_changed), eng);
	g_signal_connect (eng->proxy_port_entry, "changed",
			  G_CALLBACK (entry_changed), eng);
	g_signal_connect (eng->proxy_user_entry, "changed",
			  G_CALLBACK (entry_changed), eng);
	g_signal_connect (eng->proxy_password_entry, "changed",
			  G_CALLBACK (entry_changed), eng);

	/* Set up config control signals */
	g_signal_connect (eng->control, "apply",
			  G_CALLBACK (control_apply_cb), eng);
	g_object_weak_ref (G_OBJECT (eng->control), control_destroy_cb, eng);

	return BONOBO_OBJECT (eng->control);
}

static Bonobo_Unknown
make_factory (PortableServer_POA poa, const char *iid,
	      gpointer impl_ptr, CORBA_Environment *ev)
{
	return bonobo_shlib_factory_std (ENG_CONFIG_FACTORY_ID, poa,
					 impl_ptr, factory, NULL, ev);
}

static BonoboActivationPluginObject plugin_list[] = {
	{ ENG_CONFIG_FACTORY_ID, make_factory},
	{ NULL }
};
const  BonoboActivationPlugin Bonobo_Plugin_info = {
	plugin_list, "Evolution Non-GNOME configuration control factory"
};

